import config from '../../config';
import logger from '../logger';
import sendEmail from './mailgun';
import templates from './templates';
import { SoftError } from '../errors';

export default class EmailSender {
  /**
   * Constructor by email
   * @param {string} sender sender's email
   * @param {Object} db Sequelize instance
   */
  constructor(db, sender) {
    if (!sender) {
      this.sender = config.mailgun.from;
    } else {
      this.sender = sender;
    }
    this.db = db;
  }

  /**
   *
   * @param {string} to
   * @param {string} subject
   * @param {string} html
   */
  async send(to, subject, html) {
    if (!config.mailgun.enable) {
      return {
        id: 'idfromemailprovider'
      };
    }

    const mailOptions = {
      from: this.sender,
      to,
      subject,
      html,
    };

    try {
      const result = await sendEmail(mailOptions);
      logger.log(`The email to ${to} with subject "${subject}" was successfully sent.`);
      logger.log('Result of sending email:', result);

      return result;
    } catch (error) {
      return {};
    }
  }

  /**
   * Send email by template
   * @param {Object} options
   * @param {string} options.to
   * @param {string} options.template
   * @param {Object} options.data
   */
  async sendTemplate(options) {
    // find template
    const template = templates[options.template];
    if (!template) {
      throw new SoftError('Template not found');
    }

    logger.log(`Sending the email template with the following params: ${JSON.stringify(options)}`);

    let {html} = template;

    Object.keys(options.data).forEach((key) => {
      html = html.replace(new RegExp(`{{${key.toUpperCase()}}}`, 'g'), options.data[key]);
    });

    // set host
    html = html.replace('{{HOST}}', config.server.clientUrl);

    const subject = template.subject(options.data);

    const result = await this.send(options.to, subject, html);

    // if we get id from email provider then email is in the queue
    if (result.id) {
      await this.db.emailLog.create({
        from: this.sender,
        to: options.to,
        subject,
        template: template.name,
      });
    }
    return result;
  }
}
