const fs = require('fs');
const path = require('path');
const Sequelize = require('sequelize');
const config = require('../config/db.js');
const logger = require('../lib/logger').sqlLogger;

if (!config) {
  throw new Error('Sequelize: NODE_ENV variable is not set.');
}

module.exports = () => {
  const {
    dialect,
    port,
    host,
    database,
    timezone,
    poolSize,
    logging,
    username,
    password,
  } = config;

  if (!dialect || !host || !port || !database || !username || !timezone || !poolSize) {
    throw new Error(`Sequelize: Invalid DB parameters: ${JSON.stringify(config)}`);
  }

  const db = {};
  const sequelize = new Sequelize(database, username, password, {
    host,
    port,
    dialect,
    timezone,
    operatorsAliases: false,
    dialectOptions: {
      connectTimeout: 20000,
    },
    logging: (logging) ? (sql) => {
      logger.log(sql);
    } : null,
    pool: {
      maxConnections: poolSize,
      max: poolSize,
      minConnections: 1,
      min: 1,
    },
    define: {
      timestamps: false,
    },
  });

  fs
    .readdirSync(__dirname)
    .filter(file => (file.indexOf('.') !== 0) && (file !== 'index.js') && (file.slice(-3) === '.js'))
    .forEach((file) => {
      const model = sequelize.import(path.join(__dirname, file));
      db[model.name] = model;
    });

  Object.keys(db).forEach((modelName) => {
    if ('associate' in db[modelName]) {
      db[modelName].associate(db);
    }
  });

  db.sequelize = sequelize;
  db.Sequelize = Sequelize;

  // Associations

  // account: sessions and reset password
  db.user.hasMany(db.session);
  db.session.belongsTo(db.user);

  // schedule
  db.user.hasMany(db.schedule);
  db.schedule.belongsTo(db.user);

  db.schedule.belongsTo(db.user, { as: 'lastEditor', foreignKey: 'lastEditBy' });

  // author
  db.user.hasMany(db.appointment, { as: 'createdAppointments', foreignKey: 'authorId' });
  db.appointment.belongsTo(db.user, { as: 'author', foreignKey: 'authorId' });

  // doctor
  db.user.hasMany(db.appointment, { as: 'appointments', foreignKey: 'doctorId' });
  db.appointment.belongsTo(db.user, { as: 'doctor', foreignKey: 'doctorId' });

  db.cabinet.hasMany(db.appointment);
  db.appointment.belongsTo(db.cabinet);

  db.client.hasMany(db.appointment);
  db.appointment.belongsTo(db.client);


  return db;
};
