import moment from 'moment';
import { InvalidRequestError } from '../lib/errors';

module.exports = (sequelize, DataTypes) => {
  const Model = sequelize.define('schedule', {
    id: {
      type: DataTypes.INTEGER,
      allowNull: false,
      primaryKey: true,
      autoIncrement: true,
    },
    userId: {
      type: DataTypes.INTEGER,
      // in case of null it means it's general schedule for the whole clinic
      allowNull: true,
      references: {
        model: 'user',
        key: 'id',
      },
    },
    day: {
      type: DataTypes.ENUM('monday', 'tuesday', 'wednesday', 'thursday', 'friday', 'saturday', 'sunday'),
      allowNull: false,
    },
    // time only HH:mm:ss
    from: {
      type: DataTypes.STRING(10),
      allowNull: false,
    },
    // time only HH:mm:ss
    to: {
      type: DataTypes.STRING(10),
      allowNull: false,
    },
    lastEditBy: {
      type: DataTypes.INTEGER,
      allowNull: false,
      references: {
        model: 'user',
        key: 'id',
      },
    },
  }, {
    tableName: 'schedule',
    timestamps: true,
    updatedAt: false,
    paranoid: false,
  });

  Model.days = [
    'sunday',
    'monday',
    'tuesday',
    'wednesday',
    'thursday',
    'friday',
    'saturday',
  ];

  Model.getSchedule = async function getSchedule({ db, userId = null } = {}) {
    const rows = await db.schedule.findAll({
      where: {
        userId,
      },
    });

    if (rows.length === 0) {
      return null;
    }

    const general = rows.reduce((obj, row) => {
      const { day } = row;
      if (!obj[day]) {
        obj[day] = {
          from: '',
          to: '',
        };
      }

      obj[day].from = row.from;
      obj[day].to = row.to;

      return obj;
    }, {});

    general.isBetween = ({ day, from, to }) => {
      // no time given
      if (!from || !to) {
        return true;
      }

      // no hours for this week day in general schedule
      if (!general[day].from || !general[day].to) {
        return false;
      }

      // limits of general schedule
      const limitFrom = moment(`2000-01-01 ${general[day].from}`, 'YYYY-MM-DD hh:mm');
      const limitTo = moment(`2000-01-01 ${general[day].to}`, 'YYYY-MM-DD hh:mm');
      if (from.isBefore(limitFrom) || to.isAfter(limitTo)) {
        return false;
      }
      return true;
    };

    return general;
  };

  Model.buildPeriod = function buildPeriod({ f, t }) {
    const from = moment(`2000-01-01 ${f}`, 'YYYY-MM-DD hh:mm');
    const to = moment(`2000-01-01 ${t}`, 'YYYY-MM-DD hh:mm');

    if (!from.isValid() || !to.isValid()) {
      throw new InvalidRequestError('Wrong time format');
    }

    if (to.isBefore(from)) {
      throw new InvalidRequestError('Wrong time format');
    }

    return { from, to };
  };


  return Model;
};
