import bcrypt from 'bcryptjs';
import logger from '../lib/logger';

const saltRounds = 10;

module.exports = (sequelize, DataTypes) => {
  const User = sequelize.define('user', {
    id: {
      type: DataTypes.INTEGER,
      allowNull: false,
      primaryKey: true,
      autoIncrement: true,
    },
    email: {
      type: DataTypes.STRING,
      allowNull: false,
      unique: true,
    },
    firstName: {
      type: DataTypes.STRING,
      allowNull: false,
    },
    lastName: {
      type: DataTypes.STRING,
      allowNull: false,
    },
    password: {
      type: DataTypes.STRING,
      allowNull: false,
    },
    photoUrl: {
      type: DataTypes.STRING,
      allowNull: true,
    },
    doctorSpecialization: {
      type: DataTypes.STRING,
      allowNull: true,
    },
    role: {
      type: DataTypes.ENUM({
        values: ['director', 'doctor', 'administrator'],
      }),
      allowNull: false,
    },
    isDoctor: {
      type: DataTypes.BOOLEAN,
      allowNull: false,
    },
    colorCode: {
      type: DataTypes.STRING(50),
      allowNull: false,
    },
  }, {
    tableName: 'user',
    timestamps: true,
    paranoid: true,
  });

  User.prototype.format = function format() {
    const user = {
      id: this.id,
      email: this.email,
      role: this.role,
      firstName: this.firstName,
      lastName: this.lastName,
      photoUrl: this.photoUrl,
      doctorSpecialization: this.doctorSpecialization,
      isDoctor: this.isDoctor,
      colorCode: this.colorCode,
    };

    return user;
  };


  User.prototype.getFullName = function getFullName() {
    return `${this.firstName} ${this.lastName}`;
  };


  // --- CHECK PASSWORD ---
  User.prototype.checkPassword = function checkPassword(password) {
    return new Promise((resolve) => {
      if (!this.password) {
        resolve(false);
      }
      const res = bcrypt.compareSync(password, this.password);
      resolve(res);
    });
  };

  User.generateHash = password => new Promise((resolve) => {
    if (!password) {
      return resolve(null);
    }
    const salt = bcrypt.genSaltSync(saltRounds);
    const hash = bcrypt.hashSync(password, salt);
    resolve(hash);
  });

  // transform password to its hash
  User.addHook('beforeCreate', user => User.generateHash(user.password)
    .then((generatedHash) => {
      user.password = generatedHash;
    })
    .catch((err) => {
      logger.error(err);
      throw err;
    }));

  User.attrs = [
    'id',
    'email',
    'role',
    'firstName',
    'lastName',
    'password',
    'photoUrl',
    'doctorSpecialization',
    'isDoctor',
    'colorCode',
  ];


  return User;
};
