import { InvalidRequestError, UnauthorizedError } from '../../../lib/errors';

/**
  @api {post} /account/updatePassword Update password
  @apiGroup Account
  @apiDescription Update password

  @apiHeader {string} Authorization Bearer ${token}

  @apiParam {string} currentPassword
  @apiParam {string} newPassword

  @apiSuccessExample {json} Success
    HTTP/1.1 200 OK
    {
      "id": 5,
      "role": "doctor",
      "email": "user@test.com",
      "firstName": "Name",
      "lastName": "Last",
      "photoUrl": "",
      "doctorSpecialization": "",
      "colorCode": "",
      "isDoctor": true
    }

  @apiUse InternalError
  @apiUse UnauthorizedError
  @apiUse InvalidRequestError
*/
export default async (req, res, next) => {
  const {db, user} = req;

  req.checkBody({
    currentPassword: {
      notEmpty: true,
      errorMessage: 'Current password is required',
      isLength: {
        options: [{ min: 6 }],
      },
    },
    newPassword: {
      notEmpty: true,
      isLength: {
        options: [{ min: 6 }],
      },
      errorMessage: 'new password is required'
    },
  });

  try {
    const errs = await req.getValidationResult();
    if (!errs.isEmpty()) {
      throw new InvalidRequestError(errs.array());
    }

    const passwordMatched = await user.checkPassword(req.body.currentPassword);
    if (!passwordMatched) {
      throw new UnauthorizedError('Current password is not correct');
    }

    const hash = await db.user.generateHash(req.body.newPassword);
    await user.update({
      password: hash,
    });
    return res.send(user.format());
  } catch (err) {
    next(err);
  }
};
