import {has} from 'lodash';
import { InvalidRequestError } from '../../../lib/errors';

/**
  @api {post} /account Update user
  @apiGroup Account
  @apiDescription Update user

  @apiHeader {string} Authorization Bearer ${token}

  @apiParam {string} [firstName]
  @apiParam {string} [lastName]
  @apiParam {string} [photoUrl]
  @apiParam {number} [doctorSpecialization]
  @apiParam {number} [colorCode]

  @apiSuccessExample {json} Success
    HTTP/1.1 200 OK
    {
      "id": 5,
      "role": "doctor",
      "email": "user@test.com",
      "firstName": "Name",
      "lastName": "Last",
      "photoUrl": "",
      "doctorSpecialization": "",
      "colorCode": "",
      "isDoctor": true
    }

  @apiUse InternalError
  @apiUse UnauthorizedError
  @apiUse InvalidRequestError
*/
export default async (req, res, next) => {
  const {user} = req;
  const toUpdate = {};

  req.checkBody({
    photoUrl: {
      optional: true,
    },
    doctorSpecialization: {
      optional: true,
    },
    firstName: {
      optional: true,
    },
    lastName: {
      optional: true,
    },
  });

  try {
    const errs = await req.getValidationResult();
    if (!errs.isEmpty()) {
      throw new InvalidRequestError(errs.array());
    }

    if (has(req.body, 'firstName')) {
      toUpdate.firstName = req.body.firstName;
    }
    if (has(req.body, 'lastName')) {
      toUpdate.lastName = req.body.lastName;
    }
    if (has(req.body, 'photoUrl')) {
      toUpdate.photoUrl = req.body.photoUrl;
    }
    if (has(req.body, 'doctorSpecialization')) {
      toUpdate.doctorSpecialization = req.body.doctorSpecialization;
    }
    if (has(req.body, 'colorCode')) {
      toUpdate.colorCode = req.body.colorCode;
    }

    await user.update(toUpdate);

    return res.json(user.format());
  } catch (err) {
    next(err);
  }
};
