import moment from 'moment';

import {
  InvalidRequestError,
  NotFoundError,
  AlreadyAcceptedError,
  SoftError,
} from '../../../lib/errors';
import { upsertClient } from './helpers';

const dateFormat = 'DD-MM-YYYY HH:mm';

/**
  @api {post} /appointment Create
  @apiGroup Appointment

  @apiHeader {string} Authorization Bearer ${token}

  @apiParam {Object} appointment
  @apiParam {number} appointment.doctorId user ID
  @apiParam {number} appointment.cabinetId cabinet ID
  @apiParam {string} appointment.visitDate DD-MM-YYYY
  @apiParam {string} appointment.visitTimeFrom HH:mm
  @apiParam {string} appointment.visitTimeTo HH:mm
  @apiParam {string} appointment.complaint
  @apiParam {string} appointment.diagnosis
  @apiParam {string} appointment.comment

  @apiParam {Object} client
  @apiParam {number} [client.id] if you send ID other fields are not necessary to send
  @apiParam {string} [client.firstName]
  @apiParam {string} [client.lastName]
  @apiParam {string} [client.birthDate] DD-MM-YYYY
  @apiParam {string} [client.birthNumber]
  @apiParam {string} [client.insuranceNumber]
  @apiParam {string} [client.street]
  @apiParam {string} [client.city]
  @apiParam {string} [client.postcode]
  @apiParam {string} [client.allergies]
  @apiParam {string} [client.notificationType] "sms" || "phone" || "email"

  @apiSuccessExample {json} Success
    HTTP/1.1 200 OK
    {
      "id": 19,
      "visitDate": "25-11-2020",
      "visitTimeFrom": "15:00",
      "visitTimeTo": "16:30",
      "sessionTimeTo": "16:00",
      "complaint": "complaint complaint complaint",
      "diagnosis": "diagnosis diagnosis",
      "comment": "comment",
      "clientNotifiedAt": false,
      "author": {
          "id": 1,
          "email": "director@company.com",
          "role": "director",
          "firstName": "A",
          "lastName": "",
          "photoUrl": null,
          "doctorSpecialization": null,
          "isDoctor": true,
          "colorCode": "#000000"
      },
      "cabinet": {
          "id": 1,
          "title": "A",
          "colorCode": "#151231"
      },
      "client": {
          "id": 4,
          "firstName": "firstName",
          "lastName": "lastName",
          "birthDate": "15-02-1951",
          "birthNumber": "1234123",
          "insuranceNumber": "6662341",
          "street": "street",
          "city": "city",
          "postcode": "123122",
          "allergies": "allergies",
          "notificationType": "sms"
      },
      "doctor": {
          "id": 1,
          "email": "director@company.com",
          "role": "director",
          "firstName": "A",
          "lastName": "",
          "photoUrl": null,
          "doctorSpecialization": null,
          "isDoctor": true,
          "colorCode": "#000000"
      }
    }

  @apiUse InternalError
  @apiUse UnauthorizedError
  @apiUse InvalidRequestError
*/
export default async (req, res, next) => {
  const { db, user } = req;
  const { Op } = db.Sequelize;

  req.checkBody({
    'appointment.doctorId': {
      notEmpty: true,
      errorMessage: 'Doctor ID is required',
    },
    'appointment.cabinetId': {
      notEmpty: true,
      errorMessage: 'Cabinet ID is required',
    },
    'appointment.visitDate': {
      notEmpty: true,
      errorMessage: 'Visit date is required',
    },
    'appointment.visitTimeFrom': {
      notEmpty: true,
      errorMessage: 'Visit time From is required',
    },
    'appointment.visitTimeTo': {
      notEmpty: true,
      errorMessage: 'Visit time To is required',
    },
    'appointment.complaint': {
      notEmpty: true,
      errorMessage: 'Complaint is required',
    },
    'appointment.diagnosis': {
      notEmpty: true,
      errorMessage: 'Diagnosis is required',
    },
    'appointment.comment': {
      notEmpty: true,
      errorMessage: 'Comment is required',
    },

    'client.id': {
      optional: true,
    },
    'client.firstName': {
      optional: true,
    },
    'client.lastName': {
      optional: true,
    },
    'client.birthDate': {
      optional: true,
    },
    'client.birthNumber': {
      optional: true,
    },
    'client.insuranceNumber': {
      optional: true,
    },
    'client.street': {
      optional: true,
    },
    'client.city': {
      optional: true,
    },
    'client.postcode': {
      optional: true,
    },
    'client.allergies': {
      optional: true,
    },
    'client.notificationType': {
      optional: true,
    },
  });

  try {
    const errs = await req.getValidationResult();
    if (!errs.isEmpty()) {
      throw new InvalidRequestError(errs.array());
    }

    // validate references
    const {
      appointment: {
        doctorId,
        cabinetId,
        visitDate,
        visitTimeFrom,
        visitTimeTo,
      },
    } = req.body;
    const doctor = await db.user.findOne({
      where: {
        id: doctorId,
        role: {
          [Op.in]: ['doctor', 'director'],
        }
      },
    });
    if (!doctor) {
      throw new NotFoundError('Doctor not found');
    }
    const cabinet = await db.cabinet.findOne({
      where: {
        id: cabinetId,
      },
    });
    if (!cabinet) {
      throw new NotFoundError('Cabinet not found');
    }

    // validate dates
    const from = moment(`${visitDate} ${visitTimeFrom}`, dateFormat);
    const to = moment(`${visitDate} ${visitTimeTo}`, dateFormat);

    if (!from.isValid() || !to.isValid()) {
      throw new InvalidRequestError('Date Time format is invalid');
    }

    if (from.isSameOrBefore(moment())) {
      throw new InvalidRequestError('Time must be after current moment');
    }

    if (from.isSameOrAfter(to)) {
      throw new InvalidRequestError('Time "to" must be after the time "from"');
    }

    // is date time in range of chosen doctor's schedule
    const schedule = await db.schedule.getSchedule({ db, userId: doctor.id });
    if (!schedule) {
      throw new SoftError('Schedule is not set for doctor');
    }

    const weekDay = db.schedule.days[from.weekday()];
    const limitsFrom = moment(`${visitDate} ${schedule[weekDay].from}`, 'DD-MM-YYYY HH:mm');
    const limitsTo = moment(`${visitDate} ${schedule[weekDay].to}`, 'DD-MM-YYYY HH:mm');

    if (from.isBefore(limitsFrom) || to.isAfter(limitsTo)) {
      throw new InvalidRequestError(`Date time not in limits [${weekDay} ${schedule[weekDay].from}-${schedule[weekDay].to}]`);
    }

    // is cabinet free in this time
    const conflictedAppointment = await db.appointment.findOne({
      attributes: ['id'],
      where: {
        cabinetId: cabinet.id,
        [Op.or]: [
          { visitTimeFrom: { [Op.between]: [from.toDate(), to.toDate()] } },
          { visitTimeTo: { [Op.between]: [from.toDate(), to.toDate()] } },
        ],
      },
    });

    if (conflictedAppointment) {
      throw new AlreadyAcceptedError(`Cabinet is busy by appointment #${conflictedAppointment.id}`);
    }

    // create client and appointment
    const appointment = await db.sequelize.transaction(async (transaction) => {
      const client = await upsertClient({
        db,
        user,
        body: req.body.client,
        transaction,
      });

      return db.appointment.create({
        doctorId,
        cabinetId,
        clientId: client.id,
        visitDate: moment(req.body.appointment.visitDate, 'DD-MM-YYYY').startOf('day').toDate(),
        visitTimeFrom: from.toDate(),
        sessionTimeTo: to.toDate(),
        visitTimeTo: to.add(30, 'minute').toDate(),
        complaint: req.body.appointment.complaint,
        diagnosis: req.body.appointment.diagnosis,
        comment: req.body.appointment.comment,
        authorId: user.id,
      }, { transaction });
    });

    const row = await db.appointment.fetch({ db, id: appointment.id });
    const response = row.format();

    return res.json(response);
  } catch (err) {
    next(err);
  }
};
