import moment from 'moment';
import { has, get } from 'lodash';
import {
  InvalidRequestError,
  UnauthorizedError,
  NotFoundError,
  AlreadyAcceptedError,
  SoftError,
} from '../../../lib/errors';

const dateFormat = 'DD-MM-YYYY HH:mm';

/**
  @api {post} /appointment/:id Update
  @apiGroup Appointment

  @apiHeader {string} Authorization Bearer ${token}

  @apiParam {Object} appointment
  @apiParam {number} appointment.doctorId user ID
  @apiParam {number} appointment.cabinetId cabinet ID
  @apiParam {string} appointment.visitDate DD-MM-YYYY
  @apiParam {string} appointment.visitTimeFrom HH:mm
  @apiParam {string} appointment.visitTimeTo HH:mm
  @apiParam {string} appointment.complaint
  @apiParam {string} appointment.diagnosis
  @apiParam {string} appointment.comment

  @apiSuccessExample {json} Success
    HTTP/1.1 200 OK
    {
      "id": 19,
      "visitDate": "25-11-2020",
      "visitTimeFrom": "15:00",
      "visitTimeTo": "16:30",
      "sessionTimeTo": "16:00",
      "complaint": "complaint complaint complaint",
      "diagnosis": "diagnosis diagnosis",
      "comment": "comment",
      "clientNotifiedAt": false,
      "author": {
          "id": 1,
          "email": "director@company.com",
          "role": "director",
          "firstName": "A",
          "lastName": "",
          "photoUrl": null,
          "doctorSpecialization": null,
          "isDoctor": true,
          "colorCode": "#000000"
      },
      "cabinet": {
          "id": 1,
          "title": "A",
          "colorCode": "#151231"
      },
      "client": {
          "id": 4,
          "firstName": "firstName",
          "lastName": "lastName",
          "birthDate": "15-02-1951",
          "birthNumber": "1234123",
          "insuranceNumber": "6662341",
          "street": "street",
          "city": "city",
          "postcode": "123122",
          "allergies": "allergies",
          "notificationType": "sms"
      },
      "doctor": {
          "id": 1,
          "email": "director@company.com",
          "role": "director",
          "firstName": "A",
          "lastName": "",
          "photoUrl": null,
          "doctorSpecialization": null,
          "isDoctor": true,
          "colorCode": "#000000"
      }
    }

  @apiUse InternalError
  @apiUse UnauthorizedError
  @apiUse InvalidRequestError
*/
export default async (req, res, next) => {
  const { db, user } = req;
  const { id } = req.params;
  const { Op } = db.Sequelize;
  const up = {};

  req.checkBody({
    'appointment.doctorId': {
      optional: true,
    },
    'appointment.cabinetId': {
      optional: true,
    },
    'appointment.visitTimeFrom': {
      optional: true,
    },
    'appointment.visitTimeTo': {
      optional: true,
    },
    'appointment.complaint': {
      optional: true,
    },
    'appointment.diagnosis': {
      optional: true,
    },
    'appointment.comment': {
      optional: true,
    },
  });

  try {
    const errs = await req.getValidationResult();
    if (!errs.isEmpty()) {
      throw new InvalidRequestError(errs.array());
    }

    if (['director', 'administrator'].indexOf(user.role) === -1) {
      throw new UnauthorizedError('Access denied');
    }

    const appointment = await db.appointment.fetch({ db, id });
    if (!appointment) {
      throw new NotFoundError('Appointment not found');
    }

    // doctorId
    if (has(req.body.appointment, 'doctorId')) {
      up.doctorId = req.body.appointment.doctorId;

      const doctor = await db.user.findOne({
        attributes: ['id'],
        where: {
          id: req.body.appointment.doctorId,
          role: {
            [Op.in]: ['doctor', 'director'],
          },
        },
      });
      if (!doctor) {
        throw new NotFoundError('Doctor not found');
      }
    }

    // cabinetId
    if (has(req.body.appointment, 'cabinetId')) {
      up.cabinetId = req.body.appointment.cabinetId;

      const cabinet = await db.cabinet.findOne({
        attributes: ['id'],
        where: {
          id: req.body.appointment.cabinetId,
        },
      });
      if (!cabinet) {
        throw new NotFoundError('Cabinet not found');
      }
    }

    // visitDate
    // visitTimeFrom
    // visitTimeTo
    // sessionTimeTo
    if (
      has(req.body.appointment, 'visitDate')
      || has(req.body.appointment, 'visitTimeFrom')
      || has(req.body.appointment, 'visitTimeTo')
    ) {
      // get existing or passed values in certain format
      let visitDate = moment(appointment.visitDate).format('DD-MM-YYYY');
      let visitTimeFrom = moment(appointment.visitTimeFrom).format('HH:mm');
      let visitTimeTo = moment(appointment.visitTimeTo).format('HH:mm');

      if (has(req.body.appointment, 'visitDate')) {
        const d = moment(req.body.appointment.visitDate, 'DD-MM-YYYY');
        if (!d.isValid()) {
          throw new InvalidRequestError('Visit date is invalid');
        }
        visitDate = d.format('DD-MM-YYYY');
      }

      if (has(req.body.appointment, 'visitTimeFrom')) {
        const d = moment(`${visitDate} ${req.body.appointment.visitTimeFrom}`, 'DD-MM-YYYY HH:mm');
        if (!d.isValid()) {
          throw new InvalidRequestError('Visit date time from is invalid');
        }
        visitTimeFrom = d.format('HH:mm');
      }

      if (has(req.body.appointment, 'visitTimeTo')) {
        const d = moment(`${visitDate} ${req.body.appointment.visitTimeTo}`, 'DD-MM-YYYY HH:mm');
        if (!d.isValid()) {
          throw new InvalidRequestError('Visit date time to is invalid');
        }
        visitTimeTo = d.format('HH:mm');
      }


      // build dates back in moment instances
      const from = moment(`${visitDate} ${visitTimeFrom}`, dateFormat);
      const to = moment(`${visitDate} ${visitTimeTo}`, dateFormat);

      if (!from.isValid() || !to.isValid()) {
        throw new InvalidRequestError('Date Time format is invalid');
      }

      if (from.isSameOrBefore(moment())) {
        throw new InvalidRequestError('Time must be after current moment');
      }

      if (from.isSameOrAfter(to)) {
        throw new InvalidRequestError('Time "to" must be after the time "from"');
      }

      // is date time in range of chosen doctor's schedule
      const schedule = await db.schedule.getSchedule({
        db,
        userId: req.body.appointment.doctorId || appointment.doctorId,
      });
      if (!schedule) {
        throw new SoftError('Schedule is not set for doctor');
      }

      const weekDay = db.schedule.days[from.weekday()];
      const limitsFrom = moment(`${visitDate} ${schedule[weekDay].from}`, 'DD-MM-YYYY HH:mm');
      const limitsTo = moment(`${visitDate} ${schedule[weekDay].to}`, 'DD-MM-YYYY HH:mm');

      if (from.isBefore(limitsFrom) || to.isAfter(limitsTo)) {
        throw new InvalidRequestError(`Date time not in limits [${weekDay} ${schedule[weekDay].from}-${schedule[weekDay].to}]`);
      }


      // is cabinet free in this time
      const conflictedAppointment = await db.appointment.findOne({
        attributes: ['id'],
        where: {
          cabinetId: req.body.appointment.cabinetId || appointment.cabinetId,
          [Op.or]: [
            { visitTimeFrom: { [Op.between]: [from.toDate(), to.toDate()] } },
            { visitTimeTo: { [Op.between]: [from.toDate(), to.toDate()] } },
          ],
          id: { [Op.not]: appointment.id },
        },
      });

      if (conflictedAppointment) {
        throw new AlreadyAcceptedError(`Cabinet is busy by appointment #${conflictedAppointment.id}`);
      }

      up.visitDate = moment(visitDate, 'DD-MM-YYYY').startOf('day').toDate();
      up.visitTimeFrom = from.toDate();
      up.sessionTimeTo = to.toDate();
      up.visitTimeTo = to.add(30, 'minute').toDate();
    }


    // complaint
    if (has(req.body.appointment, 'complaint')) {
      up.complaint = req.body.appointment.complaint;
    }
    // diagnosis
    if (has(req.body.appointment, 'diagnosis')) {
      up.diagnosis = req.body.appointment.diagnosis;
    }
    // comment
    if (has(req.body.appointment, 'comment')) {
      up.comment = req.body.appointment.comment;
    }

    // clientNotified
    if (has(req.body.appointment, 'clientNotified')) {
      if (get(req.body.appointment, 'clientNotified') === true) {
        up.clientNotifiedAt = new Date();
      } else {
        up.clientNotifiedAt = null;
      }
    }

    await appointment.update(up);

    const row = await db.appointment.fetch({ db, id });
    const response = row.format();

    res.send(response);
  } catch (err) {
    return next(err);
  }
};
