import { NotFoundError, UnauthorizedError, InvalidRequestError } from '../../../lib/errors';

/**
  @api {post} /auth/signin Sign In
  @apiGroup Auth
  @apiDescription Sign in with email and password

  @apiParam {string} email
  @apiParam {string} password

  @apiSuccess {string} accessToken Bearer token of the session
  @apiSuccess {Object} user User
  @apiSuccess {number} user.id ID of the user
  @apiSuccess {string} user.email Email of the user
  @apiSuccess {string} user.role
  @apiSuccess {string} user.firstName
  @apiSuccess {string} user.lastName
  @apiSuccess {string} user.photoUrl
  @apiSuccess {string} user.doctorSpecialization

  @apiSuccessExample {json} Success
    HTTP/1.1 200 OK
    {
      "accessToken": "23a73e41-056d-43b2-9d6d-9fe1243ee14b",
      "user": {
        "id": 123,
        "email": "username@email.com",
        "role": "director",
        "firstName": "John",
        "lastName": "Doe",
        "photoUrl": "",
        "doctorSpecialization": "",
        "colorCode": "",
        "isDoctor": true
      }
    }

  @apiUse InternalError
  @apiUse InvalidRequestError
  @apiUse UnauthorizedError
*/
export default async (req, res, next) => {
  const { db } = req;

  req.checkBody({
    email: {
      notEmpty: true,
      errorMessage: 'Email is required'
    },
    password: {
      notEmpty: true,
      errorMessage: 'Password is required'
    }
  });

  try {
    const errs = await req.getValidationResult();
    if (!errs.isEmpty()) {
      throw new InvalidRequestError(errs.array());
    }

    const user = await db.user.findOne({
      where: {
        email: req.body.email.toLowerCase(),
      },
    });
    if (!user) {
      throw new NotFoundError('User not found');
    }

    const checkPassword = await user.checkPassword(req.body.password);
    if (!checkPassword) {
      throw new UnauthorizedError('Password is not correct');
    }

    const session = await db.session.create({
      userId: user.id,
      clientIP: req.clientIP,
    });

    user.session = session;

    res.send({
      token: session.token,
      user: user.format(),
    });
  } catch (error) {
    next(error);
  }
};
