import {
  InvalidRequestError,
  AlreadyRegisteredError,
} from '../../../lib/errors';

/**
  @api {post} /auth/signup Sign Up
  @apiGroup Auth
  @apiDescription Sign up new user

  @apiHeader {string} Authorization Bearer ${token} // only director can add new users

  @apiParam {string} email It must be unique in the system
  @apiParam {string} password It has to be at least 6 symbols
  @apiParam {string} role "director" || "doctor" || "administrator"

  @apiParam {string} doctorSpecialization
  @apiParam {string} firstName
  @apiParam {string} lastName
  @apiParam {string} colorCode
  @apiParam {boolean} [isDoctor] in case of new director, you should define is new director also a doctor?
  @apiParam {string} [photoUrl]

  @apiSuccessExample {json} Success
    HTTP/1.1 200 OK
    {
      "id": 123,
      "email": "username@email.com",
      "role": "doctor",
      "firstName": "John",
      "lastName": "Doe",
      "photoUrl": "",
      "doctorSpecialization": "cancer"
    }

  @apiUse InternalError
  @apiUse InvalidRequestError
  @apiUse UnauthorizedError
*/
export default async (req, res, next) => {
  const { db } = req;
  // const Op = db.Sequelize;

  /**
   * Check body
   */
  req.checkBody({
    role: {
      notEmpty: true,
      errorMessage: 'Role is required',
      isIn: {
        options: [['director', 'doctor', 'administrator']],
        errorMessage: 'Role is invalid',
      },
    },
    email: {
      notEmpty: true,
      isEmail: true,
      errorMessage: 'Invalid email',
    },
    firstName: {
      notEmpty: true,
      errorMessage: 'First Name is required',
    },
    lastName: {
      notEmpty: true,
      errorMessage: 'Last Name is required',
    },
    photoUrl: {
      optional: true,
    },
    doctorSpecialization: {
      optional: true,
    },
    password: {
      notEmpty: true,
      isLength: {
        options: [{ min: 6 }],
      },
      errorMessage: 'Invalid password',
    },
    colorCode: {
      notEmpty: true,
      errorMessage: 'Color code is required',
    },
    isDoctor: {
      optional: true,
    },
  });

  try {
    /**
     * Validation
     */
    const errs = await req.getValidationResult();
    if (!errs.isEmpty()) {
      throw new InvalidRequestError(errs.array());
    }

    /**
     * Vars
     */
    const email = req.body.email.toLowerCase();

    /**
     * check for duplicates by email
     */
    {
      const user = await db.user.findOne({
        attributes: ['id', 'email'],
        where: { email }
      });
      if (user) {
        throw new AlreadyRegisteredError('Email is busy');
      }
    }

    /**
     * Create user and sesion in one db transaction
     */
    const response = await db.sequelize.transaction(async (transaction) => {
      const createUser = {
        email,
        role: req.body.role,
        password: req.body.password,
        firstName: req.body.firstName,
        lastName: req.body.lastName,
        photoUrl: req.body.photoUrl || '',
        doctorSpecialization: req.body.doctorSpecialization || '',
        colorCode: req.body.colorCode || '',
      };

      if (req.body.role === 'director') {
        createUser.isDoctor = req.body.isDoctor || false;
      } else if (req.body.role === 'director') {
        createUser.isDoctor = true;
      } else {
        createUser.isDoctor = false;
      }

      const user = await db.user.create(createUser, { transaction });

      return user.format();
    });

    await res.json(response);
  } catch (error) {
    next(error);
  }
};
