import logger from '../../../lib/logger';
import { NotFoundError, ForbiddenError } from '../../../lib/errors';

/**
  @api {delete} /cabinet/:id Delete cabinet
  @apiGroup Cabinet

  @apiHeader {string} Authorization Bearer ${token}

  @apiSuccessExample {json} Success
    HTTP/1.1 200 OK
    {
      "success": true
    }

  @apiUse UnauthorizedError
  @apiUse InternalError
*/
export default async (req, res, next) => {
  const { db } = req;
  const { id } = req.params;
  const { Op } = db.Sequelize;

  try {
    await db.sequelize.transaction(async (transaction) => {
      const cabinet = await db.cabinet.findOne({
        where: { id },
        transaction,
      });

      if (!cabinet) {
        throw new NotFoundError('Cabinet not found');
      }

      const futureAppointments = await db.appointment.count({
        where: {
          cabinetId: cabinet.id,
          visitTimeFrom: { [Op.gt]: new Date() },
        },
        transaction,
      });
      if (futureAppointments !== 0) {
        throw new ForbiddenError('Cabinet has scheduled appointments');
      }

      await cabinet.destroy({
        transaction,
      });
    });

    return res.send({
      success: true,
    });
  } catch (err) {
    logger.error(err);
    next(err);
  }
};
