import { has } from 'lodash';
import { InvalidRequestError, NotFoundError } from '../../../lib/errors';

/**
  @api {post} /cabinet/:id Update cabinet
  @apiGroup Cabinet

  @apiHeader {string} Authorization Bearer ${token}

  @apiParam {string} [title]
  @apiParam {string} [colorCode]

  @apiSuccessExample {json} Success
    HTTP/1.1 200 OK
    {
      "id": 5,
      "title": "A",
      "colorCode": null
    }

  @apiUse InternalError
  @apiUse UnauthorizedError
  @apiUse InvalidRequestError
*/
export default async (req, res, next) => {
  const { db } = req;
  const { id } = req.params;

  req.checkBody({
    title: {
      optional: true,
    },
    colorCode: {
      optional: true,
    },
  });

  try {
    const errs = await req.getValidationResult();
    if (!errs.isEmpty()) {
      throw new InvalidRequestError(errs.array());
    }

    const cabinet = await db.cabinet.findOne({
      where: { id },
    });

    if (!cabinet) {
      throw new NotFoundError('Cabinet not found');
    }

    const toUpdate = {};

    if (has(req.body, 'title')) {
      toUpdate.title = req.body.title;
    }
    if (has(req.body, 'colorCode')) {
      toUpdate.colorCode = req.body.colorCode;
    }

    await cabinet.update(toUpdate);
    const response = cabinet.format();

    return res.send(response);
  } catch (err) {
    next(err);
  }
};
