import moment from 'moment';
import { InvalidRequestError } from '../../../lib/errors';

/**
  @api {get} /calendar/? Calendar
  @apiGroup Appointment

  @apiHeader {string} Authorization Bearer ${token}

  @apiParam {String} type "day" or "week" (query param)
  @apiParam {String} date DD-MM-YYYY (query param)

  @apiSuccessExample {json} Success
    HTTP/1.1 200 OK
    [{
      "id": 19,
      "visitDate": "25-11-2020",
      "weekday": "wednesday",
      "visitTimeFrom": "15:00",
      "visitTimeTo": "16:30",
      "sessionTimeTo": "16:00",
      "complaint": "complaint complaint complaint",
      "diagnosis": "diagnosis diagnosis",
      "comment": "comment",
      "clientNotifiedAt": false,
      "author": {
          "id": 1,
          "email": "director@company.com",
          "role": "director",
          "firstName": "A",
          "lastName": "",
          "photoUrl": null,
          "doctorSpecialization": null,
          "isDoctor": true,
          "colorCode": "#000000"
      },
      "cabinet": {
          "id": 1,
          "title": "A",
          "colorCode": "#151231"
      },
      "client": {
          "id": 4,
          "firstName": "firstName",
          "lastName": "lastName",
          "birthDate": "15-02-1951",
          "birthNumber": "1234123",
          "insuranceNumber": "6662341",
          "street": "street",
          "city": "city",
          "postcode": "123122",
          "allergies": "allergies",
          "notificationType": "sms"
      },
      "doctor": {
          "id": 1,
          "email": "director@company.com",
          "role": "director",
          "firstName": "A",
          "lastName": "",
          "photoUrl": null,
          "doctorSpecialization": null,
          "isDoctor": true,
          "colorCode": "#000000"
      }
    }]

  @apiUse InternalError
  @apiUse UnauthorizedError
*/
export default async (req, res) => {
  const { db } = req;
  const { Op } = db.sequelize;
  const where = {};
  const {
    type = 'day',
    day = moment().format('DD-MM-YYYY'),
  } = req.query;
  const date = moment(day, 'DD-MM-YYYY');

  if (!date.isValid()) {
    throw new InvalidRequestError('Day is invalid');
  }

  if (type === 'day') {
    where.visitDate = {
      [Op.between]: [
        date.startOf('day').toDate(),
        date.endOf('day').toDate(),
      ],
    };
  } else {
    where.visitDate = {
      [Op.between]: [
        date.startOf('isoWeek').toDate(),
        date.endOf('isoWeek').toDate(),
      ],
    };
  }

  const rows = await db.appointment.findAll({
    attributes: ['id'],
    where,
    order: [
      ['visitDate', 'ASC'],
    ],
  });

  const response = await Promise.map(rows, async (row) => {
    const app = await db.appointment.fetch({ db, id: row.id });
    const formatted = app.format();
    formatted.weekday = db.schedule.days[moment(row.visitDate).weekday()];

    return formatted;
  });

  res.send(response);
};
