import { InvalidRequestError } from '../../../lib/errors';


/**
  @api {post} /requestedScheduleUpdates Create request
  @apiGroup Requested Schedule

  @apiHeader {string} Authorization Bearer ${token} // only for doctors

  @apiParam {object} monday
  @apiParam {string} monday.from HH:mm
  @apiParam {string} monday.to HH:mm
  @apiParam {object} tuesday
  @apiParam {string} tuesday.from HH:mm
  @apiParam {string} tuesday.to HH:mm
  @apiParam {object} wednesday
  @apiParam {string} wednesday.from HH:mm
  @apiParam {string} wednesday.to HH:mm
  @apiParam {object} thursday
  @apiParam {string} thursday.from HH:mm
  @apiParam {string} thursday.to HH:mm
  @apiParam {object} friday
  @apiParam {string} friday.from HH:mm
  @apiParam {string} friday.to HH:mm
  @apiParam {object} saturday
  @apiParam {string} saturday.from HH:mm
  @apiParam {string} saturday.to HH:mm
  @apiParam {object} sunday
  @apiParam {string} sunday.from HH:mm
  @apiParam {string} sunday.to HH:mm

  @apiSuccessExample {json} Success
    HTTP/1.1 200 OK
    {
      "monday": {
        "from": "10:00",
        "to": "16:00"
      },
      "tuesday": {
        "from": "10:00",
        "to": "16:00"
      },
      "wednesday": {
        "from": "10:00",
        "to": "16:00"
      },
      "thursday": {
        "from": "10:00",
        "to": "16:00"
      },
      "friday": {
        "from": "10:00",
        "to": "16:00"
      },
      "saturday": {
        "from": "10:00",
        "to": "16:00"
      },
      "sunday": {
        "from": "10:00",
        "to": "16:00"
      }
    }

  @apiUse InternalError
  @apiUse UnauthorizedError
  @apiUse InvalidRequestError
*/
export default async (req, res, next) => {
  const { db, user } = req;

  const scheme = db.requestedScheduleUpdates.days.reduce((obj, day) => {
    obj[`${day}.from`] = {
      notEmpty: true,
      errorMessage: `${day} is required`,
      // options: { nullable: true },
    };
    obj[`${day}.to`] = {
      notEmpty: true,
      errorMessage: `${day} is required`,
      // options: { nullable: true },
    };

    return obj;
  }, {});

  req.checkBody(scheme);

  try {
    const errs = await req.getValidationResult();
    if (!errs.isEmpty()) {
      throw new InvalidRequestError(errs.array());
    }

    const general = await db.schedule.getSchedule({ db, userId: null });

    await db.sequelize.transaction(async (transaction) => {
      await Promise.each(db.requestedScheduleUpdates.days, async (day) => {
        const { from, to } = db.schedule.buildPeriod({
          f: req.body[day].from,
          t: req.body[day].to,
        });

        // limits of general schedule
        if (!general.isBetween({ day, from, to })) {
          throw new InvalidRequestError(`Schedule for ${day} is not in limits`);
        }

        const [row, isCreated] = await db.requestedScheduleUpdates.findOrCreate({
          defaults: {
            from: from ? from.format('HH:mm') : '',
            to: to ? to.format('HH:mm') : '',
          },
          where: {
            doctorId: user.id,
            day,
          },
          transaction,
        });

        if (!isCreated) {
          return row.update({
            from: from ? from.format('HH:mm') : '',
            to: to ? to.format('HH:mm') : '',
          }, {
            transaction,
          });
        }
      });
    });

    const response = await db.requestedScheduleUpdates.getSchedule({ db, doctorId: user.id });

    return res.send(response);
  } catch (err) {
    next(err);
  }
};
