import moment from 'moment';
import { InvalidRequestError, NotFoundError, UnauthorizedError } from '../../../lib/errors';


/**
  @api {post} /schedule/:userId Set user schedule
  @apiGroup Schedule

  @apiHeader {string} Authorization Bearer ${token}

  @apiParam {object} monday
  @apiParam {string} monday.from HH:mm
  @apiParam {string} monday.to HH:mm
  @apiParam {object} tuesday
  @apiParam {string} tuesday.from HH:mm
  @apiParam {string} tuesday.to HH:mm
  @apiParam {object} wednesday
  @apiParam {string} wednesday.from HH:mm
  @apiParam {string} wednesday.to HH:mm
  @apiParam {object} thursday
  @apiParam {string} thursday.from HH:mm
  @apiParam {string} thursday.to HH:mm
  @apiParam {object} friday
  @apiParam {string} friday.from HH:mm
  @apiParam {string} friday.to HH:mm
  @apiParam {object} saturday
  @apiParam {string} saturday.from HH:mm
  @apiParam {string} saturday.to HH:mm
  @apiParam {object} sunday
  @apiParam {string} sunday.from HH:mm
  @apiParam {string} sunday.to HH:mm

  @apiSuccessExample {json} Success
    HTTP/1.1 200 OK
    {
      "monday": {
        "from": "10:00",
        "to": "16:00"
      },
      "tuesday": {
        "from": "10:00",
        "to": "16:00"
      },
      "wednesday": {
        "from": "10:00",
        "to": "16:00"
      },
      "thursday": {
        "from": "10:00",
        "to": "16:00"
      },
      "friday": {
        "from": "10:00",
        "to": "16:00"
      },
      "saturday": {
        "from": "10:00",
        "to": "16:00"
      },
      "sunday": {
        "from": "10:00",
        "to": "16:00"
      }
    }

  @apiUse InternalError
  @apiUse UnauthorizedError
  @apiUse InvalidRequestError
*/
export default async (req, res, next) => {
  const { db, user: currentUser } = req;
  const { userId } = req.params;

  const scheme = db.schedule.days.reduce((obj, day) => {
    obj[`${day}.from`] = {
      notEmpty: true,
      errorMessage: `${day} is required`,
    };
    obj[`${day}.to`] = {
      notEmpty: true,
      errorMessage: `${day} is required`,
    };

    return obj;
  }, {});

  req.checkBody(scheme);

  try {
    const errs = await req.getValidationResult();
    if (!errs.isEmpty()) {
      throw new InvalidRequestError(errs.array());
    }

    const user = await db.user.findOne({
      attributes: ['id'],
      where: { id: userId },
    });

    if (!user) {
      throw new NotFoundError('User not found');
    }

    if (`${currentUser.id}` !== `${user.id}` && user.role !== 'director') {
      throw new UnauthorizedError('Access denied');
    }

    const general = await db.schedule.getSchedule({ db, userId: null });


    await db.sequelize.transaction(async (transaction) => {
      await Promise.each(db.schedule.days, async (day) => {
        // limits of general schedule
        const limitFrom = moment(`2000-01-01 ${general[day].from}`, 'YYYY-MM-DD hh:mm');
        const limitTo = moment(`2000-01-01 ${general[day].to}`, 'YYYY-MM-DD hh:mm');

        const from = moment(`2000-01-01 ${req.body[day].from}`, 'YYYY-MM-DD hh:mm');
        const to = moment(`2000-01-01 ${req.body[day].to}`, 'YYYY-MM-DD hh:mm');

        if (!from.isValid() || !to.isValid()) {
          throw new InvalidRequestError('Wrong time format');
        }

        if (to.isBefore(from)) {
          throw new InvalidRequestError('Wrong time format');
        }

        if (from.isBefore(limitFrom) || to.isAfter(limitTo)) {
          throw new InvalidRequestError(`Schedule for ${day} is not in limits`);
        }

        const [row, isCreated] = await db.schedule.findOrCreate({
          defaults: {
            from: from.format('HH:mm'),
            to: to.format('HH:mm'),
            lastEditBy: user.id,
          },
          where: {
            userId: user.id,
            day,
          },
          transaction,
        });

        if (!isCreated) {
          return row.update({
            from: from.format('HH:mm'),
            to: to.format('HH:mm'),
            lastEditBy: user.id,
          }, {
            transaction,
          });
        }
      });
    });

    const response = await db.schedule.getSchedule({ db, userId: user.id });

    return res.send(response);
  } catch (err) {
    next(err);
  }
};
